@@ -23,22 +23,50 @@
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 # OTHER DEALINGS IN THE SOFTWARE.
-version = '0.5.6'
+version = '0.5.7'
 
 import re
+import inspect
 import socket
 import functools
 import itertools
 import warnings
 import logging
+import sys
 import traceback
+import types
+
+PY3 = sys.version_info[0] == 3
+if PY3:
+    text_type = str
+    byte_type = bytes
+    import io
+    StringIO = io.StringIO
+
+    class Compat_Repr(object):
+        def __repr__(self):
+            return self.__str__()
+else:
+    text_type = unicode
+    byte_type = str
+    import StringIO
+    StringIO = StringIO.StringIO
+
+    class Compat_Repr(object):
+        def __repr__(self):
+            return self.__str__().encode('utf-8')
 
 from datetime import datetime
 from datetime import timedelta
-from StringIO import StringIO
-from urlparse import urlsplit, parse_qs
+try:
+    from urllib.parse import urlsplit, parse_qs
+except ImportError:
+    from urlparse import urlsplit, parse_qs
 
-from BaseHTTPServer import BaseHTTPRequestHandler
+try:
+    from http.server import BaseHTTPRequestHandler
+except ImportError:
+    from BaseHTTPServer import BaseHTTPRequestHandler
 
 old_socket = socket.socket
 old_create_connection = socket.create_connection
@@ -59,7 +87,8 @@ except ImportError:
 try:
     import ssl
     old_ssl_wrap_socket = ssl.wrap_socket
-    old_sslwrap_simple = ssl.sslwrap_simple
+    if not PY3:
+        old_sslwrap_simple = ssl.sslwrap_simple
     old_sslsocket = ssl.SSLSocket
 except ImportError:
     ssl = None
@@ -70,10 +99,10 @@ class HTTPrettyError(Exception):
 
 
 def utf8(s):
-    if isinstance(s, unicode):
+    if isinstance(s, text_type):
         s = s.encode('utf-8')
 
-    return str(s)
+    return byte_type(s)
 
 
 def parse_requestline(s):
@@ -97,7 +126,7 @@ def parse_requestline(s):
         raise ValueError('Not a Request-Line')
 
 
-class HTTPrettyRequest(BaseHTTPRequestHandler, object):
+class HTTPrettyRequest(BaseHTTPRequestHandler, Compat_Repr, object):
     def __init__(self, headers, body=''):
         self.body = utf8(body)
         self.raw_headers = utf8(headers)
@@ -108,7 +137,7 @@ class HTTPrettyRequest(BaseHTTPRequestHandler, object):
         self.method = self.command
         self.querystring = parse_qs(self.path.split("?", 1)[-1])
 
-    def __repr__(self):
+    def __str__(self):
         return 'HTTPrettyRequest(headers={0}, body="{1}")'.format(
             self.headers,
             self.body,
@@ -230,6 +259,7 @@ class fakesock(object):
             hostnames = [i.hostname for i in HTTPretty._entries.keys()]
             self.fd.seek(0)
             try:
+                print("data", data)
                 requestline, _ = data.split('\r\n', 1)
                 method, path, version = parse_requestline(requestline)
                 is_parsing_headers = True
@@ -245,7 +275,7 @@ class fakesock(object):
                     try:
                         return HTTPretty.historify_request(headers, body, False)
 
-                    except Exception, e:
+                    except Exception as e:
                         logging.error(traceback.format_exc(e))
                         return self._true_sendall(data, *args, **kw)
 
@@ -277,7 +307,17 @@ class fakesock(object):
                 self._entry = entry
 
         def debug(*a, **kw):
-            import debug
+            frame = inspect.stack()[0][0]
+            lines = map(utf8, traceback.format_stack(frame))
+
+            message = [
+                "HTTPretty intercepted and unexpected socket method call.",
+                ("Please open an issue at "
+                 "'https://github.com/gabrielfalcao/HTTPretty/issues'"),
+                "And paste the following traceback:\n",
+                "".join(lines),
+            ]
+            raise RuntimeError("\n".join(message))
 
         def settimeout(self, new_timeout):
             self.timeout = new_timeout
@@ -391,7 +431,7 @@ STATUSES = {
 }
 
 
-class Entry(object):
+class Entry(Compat_Repr, object):
     def __init__(self, method, uri, body,
                  adding_headers=None,
                  forcing_headers=None,
@@ -401,10 +441,15 @@ class Entry(object):
 
         self.method = method
         self.uri = uri
-        self.body = body
+
+        if isinstance(body, types.FunctionType):
+            self.body = body(method, uri, headers)
+        else:
+            self.body = body
+
         self.streaming = streaming
         if not streaming:
-            self.body_length = len(body or '')
+            self.body_length = len(self.body or '')
         else:
             self.body_length = 0
         self.adding_headers = adding_headers or {}
@@ -444,7 +489,7 @@ class Entry(object):
                     )
                 )
 
-    def __repr__(self):
+    def __str__(self):
         return r'<Entry %s %s getting %d>' % (
             self.method, self.uri, self.status)
 
@@ -512,7 +557,7 @@ class Entry(object):
         fk.seek(0)
 
 
-class URIInfo(object):
+class URIInfo(Compat_Repr, object):
     def __init__(self,
                  username='',
                  password='',
@@ -556,7 +601,7 @@ class URIInfo(object):
             self.current_entry += 1
         return entry
 
-    def __unicode__(self):
+    def __str__(self):
         attrs = (
             'username',
             'password',
@@ -565,16 +610,13 @@ class URIInfo(object):
             'path',
         )
         fmt = ", ".join(['%s="%s"' % (k, getattr(self, k, '')) for k in attrs])
-        return ur'<httpretty.URIInfo(%s)>' % fmt
-
-    def __repr__(self):
-        return unicode(self)
+        return r'<httpretty.URIInfo(%s)>' % fmt
 
     def __hash__(self):
-        return hash(unicode(self))
+        return hash(text_type(self))
 
     def __eq__(self, other):
-        return unicode(self) == unicode(other)
+        return text_type(self) == text_type(other)
 
     @classmethod
     def from_uri(cls, uri, entry):
@@ -590,7 +632,7 @@ class URIInfo(object):
                    entry)
 
 
-class HTTPretty(object):
+class HTTPretty(Compat_Repr, object):
     u"""The URI registration class"""
     _entries = {}
     latest_requests = []
@@ -635,7 +677,7 @@ class HTTPretty(object):
             headers['status'] = status
 
             entries_for_this_uri = [
-                cls.Response(**headers),
+                cls.Response(method=method, uri=uri, **headers),
             ]
 
         map(lambda e: setattr(e, 'uri', uri) or setattr(e, 'method', method),
@@ -647,11 +689,11 @@ class HTTPretty(object):
 
         cls._entries[info] = entries_for_this_uri
 
-    def __repr__(self):
+    def __str__(self):
         return u'<HTTPretty with %d URI entries>' % len(self._entries)
 
     @classmethod
-    def Response(cls, body, adding_headers=None, forcing_headers=None,
+    def Response(cls, body, method=None, uri=None, adding_headers=None, forcing_headers=None,
                  status=200, streaming=False, **headers):
 
         headers['body'] = body
@@ -659,7 +701,7 @@ class HTTPretty(object):
         headers['forcing_headers'] = forcing_headers
         headers['status'] = int(status)
         headers['streaming'] = streaming
-        return Entry(method=None, uri=None, **headers)
+        return Entry(method, uri, **headers)
 
     @classmethod
     def disable(cls):
@@ -689,12 +731,14 @@ class HTTPretty(object):
 
         if ssl:
             ssl.wrap_socket = old_ssl_wrap_socket
-            ssl.sslwrap_simple = old_sslwrap_simple
             ssl.SSLSocket = old_sslsocket
             ssl.__dict__['wrap_socket'] = old_ssl_wrap_socket
-            ssl.__dict__['sslwrap_simple'] = old_sslwrap_simple
             ssl.__dict__['SSLSocket'] = old_sslsocket
 
+            if not PY3:
+                ssl.sslwrap_simple = old_sslwrap_simple
+                ssl.__dict__['sslwrap_simple'] = old_sslwrap_simple
+
     @classmethod
     def enable(cls):
         socket.socket = fakesock.socket
@@ -723,13 +767,15 @@ class HTTPretty(object):
 
         if ssl:
             ssl.wrap_socket = fake_wrap_socket
-            ssl.sslwrap_simple = fake_wrap_socket
             ssl.SSLSocket = FakeSSLSocket
 
             ssl.__dict__['wrap_socket'] = fake_wrap_socket
-            ssl.__dict__['sslwrap_simple'] = fake_wrap_socket
             ssl.__dict__['SSLSocket'] = FakeSSLSocket
 
+            if not PY3:
+                ssl.sslwrap_simple = fake_wrap_socket
+                ssl.__dict__['sslwrap_simple'] = fake_wrap_socket
+
 
 def httprettified(test):
     "A decorator tests that use HTTPretty"
